@@ -4,7 +4,7 @@ use File::Path 'mkpath';
 use File::Copy 'copy';
 use Config;
 
-my $VERSION = '1.07';			# Changelog at end
+my $VERSION = '1.08';			# Changelog at end
 die "Debugging cycle detected"		# set to -1 to allow extra iteration
   if ++$ENV{PERL_DEBUG_MCODE_CYCLE} > 1;
 
@@ -213,6 +213,8 @@ sub do_subdir_build () {
   $make = 'make' unless defined $make;
   system $make and die "system($make): rc=$?";
   { open my $f, '>', 'autodebug-make-ok'; }	# Leave a footprint of a successful build
+  warn "Renaming Makefile.PL to orig-Makefile.PL\n\t(to avoid recursive calls from Makefile.PL in the parent directory)";
+  rename 'Makefile.PL', 'orig-Makefile.PL'; # ignore error
 }
 
 do_subdir_build() unless -f 'autodebug-make-ok';
@@ -397,3 +399,5 @@ __END__
 1.07	Update the error message if no debugger found.
 	Inspect dbxtool as a way of debugging ???.  (untested)
 	Report other debugger candidates if a run of a debugger fails.
+1.08	Rename Makefile.PL in the subdirectory (to avoid recursive calls from Makefile.PL in the parent directory).
+	Remove the corresponding warning from ./README too.
\ No newline at end of file
